@@ -6,14 +6,16 @@
 ##' to check the compatibility of the data.
 ##' 
 ##' @param ... objects which element names to compare
-##' @param keepNames If TRUE, the original dataset names are used in
+##' @param list.data As alternative to ..., you can supply the data
+##'     sets in a list here.
+##' @param keep.names If TRUE, the original dataset names are used in
 ##'     reported table. If not, generic x1, x2,... are used. The
 ##'     latter may be preferred for readability.
 ##' @param testEqual Do you just want a TRUE/FALSE to whether the
 ##'     names of the two objects are the same? Default is FALSE which
 ##'     means to return an overview for interactive use. You might
 ##'     want to use TRUE in programming. However, notice that this
-##'     check may be overly rigorous. Many classes are compitable
+##'     check may be overly rigorous. Many classes are compatible
 ##'     enough (say numeric and integer), and compareCols doesn't take
 ##'     this into account.
 ##' @param diff.only If TRUE, don't report columns where no difference
@@ -35,21 +37,22 @@
 ##'     on what data is found. But consider setting this to TRUE for
 ##'     non-interactive use. Default can be configured using
 ##'     NMdataConf.
-##' @param as.fun A function that will be run un the result before
+##' @param as.fun A function that will be run on the result before
 ##'     returning. If first input data set is a data.table, the
 ##'     default is to return a data.table, if not the default is to
 ##'     return a data.frame. Use whatever to get what fits in with
 ##'     your workflow. Default can be configured with NMdataConf.
-##' @details tecnically, this function compares classes of elements in
+##' @details technically, this function compares classes of elements in
 ##'     lists. However, in relation to NMdata, this will most of the
 ##'     time be columns in data.frames.
-##' @return A data.frame with an overview of elementes and their
+##' @param keepNames Deprecated. Use keep.names instead.
+##' @return A data.frame with an overview of elements and their
 ##'     classes of objects in ... Class as defined by as.fun.
 ##' @family DataWrangling
 ##' @export
 
 
-compareCols <- function(...,keepNames=TRUE,testEqual=FALSE,diff.only=TRUE,cols.wanted,fun.class=base::class,quiet,as.fun){
+compareCols <- function(...,list.data,keep.names=TRUE,testEqual=FALSE,diff.only=TRUE,cols.wanted,fun.class=base::class,quiet,as.fun,keepNames){
     
 
 #### Section start: Dummy variables, only not to get NOTE's in pacakge checks ####
@@ -64,19 +67,33 @@ compareCols <- function(...,keepNames=TRUE,testEqual=FALSE,diff.only=TRUE,cols.w
 
     if(missing(quiet)) quiet <- NULL
     quiet <- NMdataDecideOption("quiet",quiet)
-    dots <- list(...)
+
+    if(!missing(keepNames)){
+        message("compareCols: keepNames argument is deprecated. Use keep.names instead. For now, keepNames is overruling keep.names.")
+        keep.names <- keepNames
+    }
+    
+    if(missing(list.data)){
+        dots <- list(...)
+        if(keep.names){
+            names.dots <- setdiff(as.character(match.call(expand.dots=TRUE)),as.character(match.call(expand.dots=FALSE)))
+        }
+    } else {
+        dots <- list.data
+        names.dots <- names(dots)
+    }
+
+    
     ndots <- length(dots)
     if(ndots==0) stop("No data supplied.")
     if(ndots==1&&missing(diff.only)) diff.only <- FALSE
     if(missing(cols.wanted)) cols.wanted <- NULL
-    
-    if(keepNames){
-        names.dots <- setdiff(as.character(match.call(expand.dots=TRUE)),as.character(match.call(expand.dots=FALSE)))
-    } else {
+
+
+    if(!keep.names){
         names.dots <- paste0("x",seq(ndots))
     }
     names(dots) <- names.dots
-
     
     
     df1.was.dt <- is.data.table(dots[[1]])
@@ -107,7 +124,8 @@ compareCols <- function(...,keepNames=TRUE,testEqual=FALSE,diff.only=TRUE,cols.w
 
     ## merge back on
     dt.cols <- mergeCheck(dt.cols,nu.classes,by="column",quiet=TRUE)
-
+    
+    
     if(testEqual) return(dt.cols[n<ndots|nu>1,.N]==0)
 
     if(is.null(cols.wanted)){
@@ -116,6 +134,9 @@ compareCols <- function(...,keepNames=TRUE,testEqual=FALSE,diff.only=TRUE,cols.w
         dt.cols <- merge(dt.cols,dt.wanted,all=T)
         dt.cols[!is.na(get(col.wanted)),column:=paste0("*",column)]
     }
+
+    dt.cols.full <- copy(dt.cols)
+    
     ## criteria whether to show if nu=1 (all equal class)
     if(diff.only) dt.cols <- dt.cols[n<ndots|nu>1|get(col.wanted)>0]
 ### this one orders by number of occurance, unique classses, column name
@@ -132,8 +153,10 @@ compareCols <- function(...,keepNames=TRUE,testEqual=FALSE,diff.only=TRUE,cols.w
 ### what about a summary of dimensions of the supplied datasets? We do that in the separate "dims" function.
     
     if(!quiet) {
-        message("Dimensions:")
-        print(dims(list.data=dots))
+        if(all(sapply(dots,is.data.frame))){
+            message("Dimensions:")
+            print(dims(list.data=dots))
+        }
 
         if(nrow(dt.cols)==0&&ndots==1){
             message("Only one data set supplied.\n")
@@ -142,10 +165,13 @@ compareCols <- function(...,keepNames=TRUE,testEqual=FALSE,diff.only=TRUE,cols.w
         } else {
             if(diff.only){
                 message("\nColumns that differ:")
+                print(dt.cols)
+                message()
+                messageWrap(paste0("\nColumns where no differences were found: ",paste(dt.cols.full[nu==1&n==ndots,column],collapse=", "),"."),fun.msg=message)
             } else {
                 message("\nOverview of all columns:")
+                print(dt.cols)
             }
-            print(dt.cols)
         }
         return(invisible(as.fun(dt.cols)))
     } else {
