@@ -17,9 +17,10 @@ test_that("basic",{
 
     pk <- readRDS(file=system.file("examples/data/xgxr2.rds",package="NMdata"))
 
-    res1 <- NMwriteData(pk,file="testOutput/NMwriteData1.csv",
-                        write.rds=F,write.csv=F,nmdir.data="/example")
-    res1 <- fix.input(res1)
+    NMwriteData(pk,file="testOutput/NMwriteData1.csv",
+                        write.rds=F,write.csv=T,nmdir.data="/example")
+    res1 <- readLines("testOutput/NMwriteData1.csv")
+
     ## lapply(res1,print)
     ## lapply(readRDS(fileRef),print)
 
